#!/usr/bin/env node
'use strict';

var colorize = require('json-colorizer');
var jsonata = require('jsonata');
var readInput = require('read-input');
var fileExists = require('file-exists');
var program = require('commander');
var readFilePromise = require('fs-readfile-promise');
var YAML = require('js-yaml');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var colorize__default = /*#__PURE__*/_interopDefaultLegacy(colorize);
var jsonata__default = /*#__PURE__*/_interopDefaultLegacy(jsonata);
var readInput__default = /*#__PURE__*/_interopDefaultLegacy(readInput);
var fileExists__default = /*#__PURE__*/_interopDefaultLegacy(fileExists);
var program__default = /*#__PURE__*/_interopDefaultLegacy(program);
var readFilePromise__default = /*#__PURE__*/_interopDefaultLegacy(readFilePromise);
var YAML__default = /*#__PURE__*/_interopDefaultLegacy(YAML);

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }
  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}
function _asyncToGenerator(fn) {
  return function () {
    var self = this,
      args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);
      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }
      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }
      _next(undefined);
    });
  };
}
function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}
function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;
  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];
  return arr2;
}
function _createForOfIteratorHelper(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];
  if (!it) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;
      var F = function () {};
      return {
        s: F,
        n: function () {
          if (i >= o.length) return {
            done: true
          };
          return {
            done: false,
            value: o[i++]
          };
        },
        e: function (e) {
          throw e;
        },
        f: F
      };
    }
    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }
  var normalCompletion = true,
    didErr = false,
    err;
  return {
    s: function () {
      it = it.call(o);
    },
    n: function () {
      var step = it.next();
      normalCompletion = step.done;
      return step;
    },
    e: function (e) {
      didErr = true;
      err = e;
    },
    f: function () {
      try {
        if (!normalCompletion && it.return != null) it.return();
      } finally {
        if (didErr) throw err;
      }
    }
  };
}

var getopts = ( /*#__PURE__*/(function () {
  var _ref = _asyncToGenerator(function* (argv) {
    const options = program__default["default"].option('-n, --ndjson', 'Newline Delimited JSON').option('-j, --json', 'Force JSON output').option('-y, --yaml', 'YAML output').option('-a, --accept-yaml', 'YAML input').option('-p, --plain-text', 'Do not decorate output').option('-q, --query-file <path>', 'JSONata query file').parse(argv).opts();
    const ndjson = !!options.ndjson;
    const json = !!options.json;
    const yamlOut = !!options.yaml;
    const yamlIn = !!options.acceptYaml;
    const plainText = !!options.plainText;
    const queryFile = options.queryFile;
    const files = program__default["default"].args.slice(0);
    const query = yield getQuery(queryFile, files);
    return {
      query,
      files,
      ndjson,
      json,
      yamlOut,
      yamlIn,
      plainText
    };
  });
  return function (_x) {
    return _ref.apply(this, arguments);
  };
})());
const exists = /*#__PURE__*/function () {
  var _ref2 = _asyncToGenerator(function* (path) {
    try {
      return yield fileExists__default["default"](path);
    } catch (err) {
      return false;
    }
  });
  return function exists(_x2) {
    return _ref2.apply(this, arguments);
  };
}();
const getQuery = /*#__PURE__*/function () {
  var _ref3 = _asyncToGenerator(function* (queryFile, files) {
    if (typeof queryFile === 'string' && queryFile.length) {
      return readFilePromise__default["default"](queryFile, 'utf8');
    } else {
      // If the first argument is a file that exists, then it was not a query string
      // If the first argument is not defined, then it was not a query string
      const isNotQuery = files[0] ? yield exists(files[0]) : true;
      return isNotQuery ? '$' : files.shift();
    }
  });
  return function getQuery(_x3, _x4) {
    return _ref3.apply(this, arguments);
  };
}();

const main = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator(function* () {
    const _yield$getopts = yield getopts(process.argv),
      files = _yield$getopts.files,
      ndjson = _yield$getopts.ndjson,
      json = _yield$getopts.json,
      yamlIn = _yield$getopts.yamlIn,
      yamlOut = _yield$getopts.yamlOut,
      query = _yield$getopts.query,
      plainText = _yield$getopts.plainText;
    const evaluator = parseQuery(query);
    const data = yield readInput__default["default"](files);
    var _iterator = _createForOfIteratorHelper(data.files),
      _step;
    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        const file = _step.value;
        if (file.error) {
          throw file.error;
        } else {
          const input = yamlIn ? parseYaml(file.data, file.name) : parseJson(file.data, file.name);
          const result = yield evaluator.evaluate(input);
          const output = yamlOut ? formatYaml(result) : formatJson(result, ndjson, json, plainText);
          console.log(output);
        }
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }
  });
  return function main() {
    return _ref.apply(this, arguments);
  };
}();
const parseQuery = query => {
  try {
    return jsonata__default["default"](query);
  } catch (err) {
    throw new Error('Failed to compile JSONata expression: ' + err.message);
  }
};
const formatJson = (data, ndjson, json, plainText) => {
  if (typeof data === 'undefined') {
    return '';
  }
  if (!json) {
    if (typeof data === 'string') {
      return data;
    }
    if (isSimpleArray(data)) {
      return data.join('\n');
    }
  }
  const formatted = ndjson ? JSON.stringify(data) : JSON.stringify(data, null, 2);
  return plainText ? formatted : colorize__default["default"](formatted);
};

// Is it an array containing only simple types
const isSimpleArray = arr => Array.isArray(arr) && !arr.some(i => typeof i !== 'string' && typeof i !== 'number');
const parseYaml = (string, fileName) => {
  try {
    return YAML__default["default"].load(string, {
      json: true
    });
  } catch (err) {
    if (fileName) {
      throw new Error(err.message + ' in file ' + fileName);
    } else {
      throw err;
    }
  }
};
const formatYaml = yaml => typeof yaml === 'undefined' ? '' : YAML__default["default"].dump(yaml);
const parseJson = (string, fileName) => {
  try {
    return JSON.parse(string);
  } catch (err) {
    if (fileName) {
      throw new Error(err.message + ' in file ' + fileName);
    } else {
      throw err;
    }
  }
};
main().catch(err => {
  console.error(err.message);
  process.exit(1);
});
